package models;

import java.util.Observable;
import java.util.Random;

import specifications.DominoSpecification;
import tools.InvariantBrokenException;

/**
 * Wrapper for a {@link DominoSpecification} that makes it {@link Observable}.<br>
 * This is a work-around {@link Observable} being an abstract class and not an interface 
 * (a problem with the Java API design).<br><br>
 * 
 * It<b> implements and extends </b> the {@link DominoSpecification} behaviour with 
 * a randomize method, which is added in order to allow the construction of a new random 
 * {@link DominoSpecification} attribute without having to reconstruct the Observable object
 * @author jpaulgibson
 * @version 1
 */
public class ObservableDomino extends Observable implements DominoSpecification{
	
	DominoSpecification domino;

	public ObservableDomino(int l, int r) throws IllegalArgumentException {
		
		super();
		domino = new Domino(l,r);
	}
	
	public ObservableDomino(){
		
		super();
		domino = new Domino();
	}
	
	public ObservableDomino(Random rng){
		
		super();
		domino = new Domino(rng);
	}

	@Override
	public void switchSides() {
		
		domino.switchSides();
		setChanged(); notifyObservers();
	}

	@Override
	public int getRightValue() {
		
		return domino.getRightValue();
	}

	@Override
	public int getLeftValue() {
		
		return domino.getLeftValue();
	}

	@Override
	public boolean invariant() {
		
		return (domino!=null && domino.invariant());
	}

	
	@Override
	public String toString(){
		
		return domino.toString();
	}

	@Override
	public DominoSpecification copy(){
		
		return new ObservableDomino(domino.getLeftValue(), domino.getRightValue());
	}
	
	public boolean equals (Object o){
		
		return domino.equals(o);
	}

	@Override
	public int hashCode(){
		
		return domino.hashCode();
	}	
		
	@Override
	public void randomize (){
			
		domino = new Domino(new Random());
		setChanged(); notifyObservers();
	}

	@Override
	public void randomize(Random rng) throws InvariantBrokenException {
		
			domino = new Domino(rng);   
			setChanged(); notifyObservers();	
	}


}
