package specifications;

import tools.InvariantBrokenException;
import tools.HasInvariant;
import tools.Randomizable;

/**
 * A simple domino from a traditional set:
 * <ul>
 * <li> 0:0 0:1 0:2 0:3 0:4 0:5 0:6 </li>
 * <li>     1:1 1:2 1:3 1:4 1:5 1:6 </li>
 * <li>         2:2 2:3 2:4 2:5 2:6 </li>
 * <li>             3:3 3:4 3:5 3:6 </li>
 * <li>                 4:4 4:5 4:6 </li>
 * <li>                     5:5 5:6 </li>
 * <li>                         6:6 </li>
 * </ul>
 * <b>Note</b>: we consider domino x:y to be equal to domino y:x, 
 * and must override {@link DominoSpecification#equals} and {@link DominoSpecification#hashCode} methods appropriately<br>
 * Tested by {@link tests.JUnit_DominoSpecification}
 * @author jpaulgibson
 * @version 1
 */
public interface DominoSpecification extends HasInvariant, Randomizable{
	
	/**
	 * The minimum value of a side of a domino in a standard domino set<br>
	 * <b> NOTE: </b> May be changed provided the class invariant remains true
	 */
	public static final int MIN =0;
	
	/**
	 * The maximum value of a side of a domino in a standard domino set<br>
	 * <b> NOTE: </b> May be changed provided the class invariant remains true
	 */
	public static final int MAX =6;
	
	/**
	 * The range of domino values<br>
	 * <b> NOTE: </b> Should NOT be changed as it is a derived constant
	 */
	public static final int RANGE_OF_VALUES = MAX - MIN +1;
	
	/**
	 * The total number of dominoes = 1 + 2 + 3 + ... + (RANGE_OF_VALUES)<br>
	 * <b> NOTE: </b> Should NOT be changed as it is a derived constant
	 */
	public static final int NUMBER_OF_DOMINOES_POSSIBLE = ((RANGE_OF_VALUES)*(RANGE_OF_VALUES+1))/2;
	
	/**
	 * The class invariant checks that the MAX value is bigger than the MIN value<br>
	 * <b> NOTE: </b> Should NOT be changed
	 */
	public static final boolean INVARIANT = (MAX > MIN);
	
	/**
	 * Should check that the left and right values are in allowed range 
	 * {@link DominoSpecification#MIN} .. {@link DominoSpecification#MAX}
	 * @return if the domino is in a valid state
	 */
	public boolean invariant() throws InvariantBrokenException;
	
	
	/**
	 * Tested by {@link tests.JUnit_DominoSpecification#test_equals}
	 * @param obj is compared to this
	 * @return false if the object is null or not a domino, and
	 *         true if the two values of both dominoes are the same (even if we switch/flip either domino)
	 * @see DominoSpecification#hashCode
	 */
	public boolean equals(Object obj);
	
	/**
	 * Tested by {@link tests.JUnit_DominoSpecification#test_hashCode}
	 * @return a hash value which is consistent with the {@link DominoSpecification#equals} method
	 * @see DominoSpecification#equals
	 */
	public int hashCode();
	
	/**
	 * Tested by {@link tests.JUnit_DominoSpecification#test_toString}
	 * @return String representation of domino using " : " to separate the left and right integer value characters
	 */
	public String toString();
	
	/**
	 * Constructs a new domino using a copy of the left and right values of <code> this </code><br>
	 * Tested by {@link tests.JUnit_DominoSpecification#test_copy}
	 * @return a copy of <code> this </code> 
	 */
	public DominoSpecification copy ();
	
	
	/**
	 * Swap the left and right values - to model flipping/switching the domino around. <br>
	 * Tested by {@link tests.JUnit_DominoSpecification#test_switchSides}
	 * @throws InvariantBrokenException if the state change produces an invalid domino
	 */
	public void switchSides() throws InvariantBrokenException;
	
	
	/**
	 * Getter function of the right value.<br>
	 * Tested by {@link tests.JUnit_DominoSpecification#test_getRight}
	 * @return the right value
	 */
	public int getRightValue();
	
	/**
	 * Getter function of the left value.<br>
	 * Tested by {@link tests.JUnit_DominoSpecification#test_getLeft}
	 * @return the left value
	 */
	public int getLeftValue();

}
