package views;

import java.awt.Image;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import tools.InvariantBrokenException;

/**
 * The class for downloading and images files and storing them for later use by the
 * GUI of the dominoes applications<br>
 * The images need to be stored in a subdirectory called "imagesDominos" and each file must be named:
 * <ul>
 * <li> "0-0.jpg"
 * <li> "0-1.jpg"
 * <li> ...
 * <li> "6-6.jpg"
 * </ul>
 * The size of the images is 70*35 pixels.<br>
 * The class  is implemented following the singleton design pattern, where we use a synchronized 
 * getInstance method to permit safe multi-threaded access  to the instance whilst it is being created initially.
 * @author jpaulgibson
 * @version 1
 */
public class DominoImages {
	
	/**
	 * The protected reference to the one and only instance of the Domino Images class.<br>
	 */
	protected static DominoImages uniqueInstance = null;
	
	
	public static final int IMAGE_WIDTH = 70;
	public static final int IMAGE_HEIGHT = 35;
	public static final int DOMINO_MAX = 6;
	public static final String IMAGES_DIRECTORY ="imagesDominos";
	
	private Image [][] dominoImages;
	
	private DominoImages () throws InvariantBrokenException{
		
		File imageFile = null;
		
		dominoImages = new Image [DOMINO_MAX+1][DOMINO_MAX+1];
		
	    for(int left = 0; left <= DOMINO_MAX; left++)
		  for (int right =0; right <= DOMINO_MAX; right++){
	           imageFile = new File (IMAGES_DIRECTORY+"/"+left+'-'+right+".jpg");
                try {
		          dominoImages[left][right] = ImageIO.read(imageFile);
                 } catch (IOException e) {
		                System.out.println("Could not find specified image file for domino "+ left+":"+right);
		                e.printStackTrace();
	                    }
		     }
	    if (!invariant()) throw new InvariantBrokenException ("The domino images have not been initialised properly");
	}
	
	/**
	 * @return the unique instance
	 */
	public synchronized static DominoImages instance() {
	if(uniqueInstance == null) uniqueInstance = new DominoImages();
	return uniqueInstance;
	}
	
	/**
	 * @param left is one of the values on the domino
	 * @param right is the other value on the domino
	 * @return the image corresponding to the specified domino
	 */
	public Image getDominoImage (int left, int right){
		
	       return dominoImages [left][right];
	}
	
	
	/**
	 * @return all images are fully loaded and they have the correct size
	 */
	public boolean invariant(){
		
		for(int left = 0; left <= DOMINO_MAX; left++)
			  for (int right =0; right <= DOMINO_MAX; right++){
				  if (dominoImages[left][right] == null) return false;
				  if (dominoImages[left][right].getWidth(null)!= IMAGE_WIDTH && 
				      dominoImages[left][right].getHeight(null)!= IMAGE_HEIGHT) return false;
			  }
		
		return true;
		
	}
	
}
